"""Incorrectly implemented interfaces are reported"""

from zope.interface import implementer
from zope.interface import Interface
from zope.interface import classImplements


class IFoo(Interface):
    def foo() -> int:
        pass


class Foo:
    def bar(self) -> int:
        return 0


classImplements(Foo, IFoo)


foo: IFoo = Foo()


"""
<output>
classimplements_wrong.py:18: error: 'Foo' is missing following 'IFoo' interface members: foo.
</output>
"""



# Stubs for zope.interface.exceptions (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Invalid(Exception): ...

class DoesNotImplement(Invalid):
    interface: Any = ...
    def __init__(self, interface: Any) -> None: ...

class BrokenImplementation(Invalid):
    interface: Any = ...
    name: Any = ...
    def __init__(self, interface: Any, name: Any) -> None: ...

class BrokenMethodImplementation(Invalid):
    method: Any = ...
    mess: Any = ...
    def __init__(self, method: Any, mess: Any) -> None: ...

class InvalidInterface(Exception): ...
class BadImplements(TypeError): ...



# Stubs for zope.interface.adapter (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from zope.interface._zope_interface_coptimizations import LookupBase, VerifyingBase

class BaseAdapterRegistry:
    __bases__: Any = ...
    def __init__(self, bases: Any = ...) -> None: ...
    def changed(self, originally_changed: Any) -> None: ...
    def register(self, required: Any, provided: Any, name: Any, value: Any): ...
    def registered(self, required: Any, provided: Any, name: Any = ...): ...
    def unregister(self, required: Any, provided: Any, name: Any, value: Optional[Any] = ...): ...
    def subscribe(self, required: Any, provided: Any, value: Any) -> None: ...
    def unsubscribe(self, required: Any, provided: Any, value: Optional[Any] = ...): ...
    def get(self, _: Any): ...

class LookupBaseFallback:
    def __init__(self) -> None: ...
    def changed(self, ignored: Optional[Any] = ...) -> None: ...
    def lookup(self, required: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...): ...
    def lookup1(self, required: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...): ...
    def queryAdapter(self, object: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...): ...
    def adapter_hook(self, provided: Any, object: Any, name: Any = ..., default: Optional[Any] = ...): ...
    def lookupAll(self, required: Any, provided: Any): ...
    def subscriptions(self, required: Any, provided: Any): ...
LookupBasePy = LookupBaseFallback
LookupBase = LookupBaseFallback

class VerifyingBaseFallback(LookupBaseFallback):
    def changed(self, originally_changed: Any) -> None: ...
    def lookupAll(self, required: Any, provided: Any): ...
    def subscriptions(self, required: Any, provided: Any): ...
VerifyingBasePy = VerifyingBaseFallback
VerifyingBase = VerifyingBaseFallback

class AdapterLookupBase:
    def __init__(self, registry: Any) -> None: ...
    def changed(self, ignored: Optional[Any] = ...) -> None: ...
    def init_extendors(self) -> None: ...
    def add_extendor(self, provided: Any) -> None: ...
    def remove_extendor(self, provided: Any) -> None: ...
    def queryMultiAdapter(self, objects: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...): ...
    def names(self, required: Any, provided: Any): ...
    def subscribers(self, objects: Any, provided: Any): ...

class AdapterLookup(AdapterLookupBase, LookupBase): ...

class AdapterRegistry(BaseAdapterRegistry):
    LookupClass: Any = ...
    def __init__(self, bases: Any = ...) -> None: ...
    def changed(self, originally_changed: Any) -> None: ...

class VerifyingAdapterLookup(AdapterLookupBase, VerifyingBase): ...

class VerifyingAdapterRegistry(BaseAdapterRegistry):
    LookupClass: Any = ...



# Stubs for zope.interface.document (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def asStructuredText(I: Any, munge: int = ..., rst: bool = ...): ...
def asReStructuredText(I: Any, munge: int = ...): ...



# Stubs for zope.interface.verify (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

MethodTypes: Any

def verifyClass(iface: Any, candidate: Any, tentative: int = ...): ...
def verifyObject(iface: Any, candidate: Any, tentative: int = ...): ...



# Stubs for zope.interface._compat (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

CLASS_TYPES: Any
STRING_TYPES: Any
PYTHON3: bool
PYTHON2: bool



# Stubs for zope.schema.vocabulary (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class SimpleTerm:
    value: Any = ...
    token: Any = ...
    title: Any = ...
    def __init__(self, value: Any, token: Optional[Any] = ..., title: Optional[Any] = ...) -> None: ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __hash__(self): ...

class SimpleVocabulary:
    by_value: Any = ...
    by_token: Any = ...
    def __init__(self, terms: Any, *interfaces: Any, **kwargs: Any) -> None: ...
    @classmethod
    def fromItems(cls, items: Any, *interfaces: Any): ...
    @classmethod
    def fromValues(cls, values: Any, *interfaces: Any): ...
    @classmethod
    def createTerm(cls, *args: Any): ...
    def __contains__(self, value: Any): ...
    def getTerm(self, value: Any): ...
    def getTermByToken(self, token: Any): ...
    def __iter__(self): ...
    def __len__(self): ...
    def __eq__(self, other: Any): ...
    def __ne__(self, other: Any): ...
    def __hash__(self): ...

class TreeVocabulary:
    terms_factory: Any = ...
    path_by_value: Any = ...
    term_by_value: Any = ...
    term_by_token: Any = ...
    def __init__(self, terms: Any, *interfaces: Any) -> None: ...
    def __contains__(self, value: Any): ...
    def __getitem__(self, key: Any): ...
    def __iter__(self): ...
    def __len__(self): ...
    def get(self, key: Any, default: Optional[Any] = ...): ...
    def keys(self): ...
    def values(self): ...
    def items(self): ...
    @classmethod
    def fromDict(cls, dict_: Any, *interfaces: Any): ...
    def getTerm(self, value: Any): ...
    def getTermByToken(self, token: Any): ...
    def getTermPath(self, value: Any): ...

class VocabularyRegistryError(LookupError):
    name: Any = ...
    def __init__(self, name: Any) -> None: ...

class VocabularyRegistry:
    def __init__(self) -> None: ...
    def get(self, context: Any, name: Any): ...
    def register(self, name: Any, factory: Any) -> None: ...

def getVocabularyRegistry(): ...
def setVocabularyRegistry(registry: Any) -> None: ...



"""A simple valid interface declaration
"""
import zope.interface


class ISomething(zope.interface.Interface):
    def hello(x: int, y: str) -> None:
        pass

@zope.interface.implementer(ISomething)
class AbstractSomething(object):
    def thefunc(self) -> None:
        pass
    pass

@zope.interface.implementer(ISomething)
class ConcreteSomething(object):
    def hello(self, x: int, y: str) -> None:
        pass

class Context(object):
    pass

def main() -> None:
    ctx = Context()
    smth = ISomething(ctx)
    smth.hello(2, 3)  # Error: second argument is expected to be string

    asmth = AbstractSomething()  # Error, cannot instantiate abstract class

    csmth = ConcreteSomething()  # Can instantiate (using default object constructor)

if __name__ == '__main__':
    main()

"""
<output>
adaptation.py:11: error: 'AbstractSomething' is missing following 'ISomething' interface members: hello.
adaptation.py:27: error: Argument 2 to "hello" of "ISomething" has incompatible type "int"; expected "str"
</output>
"""



# Stubs for zope.interface.common.idatetime (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from zope.interface import Interface

class ITimeDeltaClass(Interface):
    min: Any = ...
    max: Any = ...
    resolution: Any = ...

class ITimeDelta(ITimeDeltaClass):
    days: Any = ...
    seconds: Any = ...
    microseconds: Any = ...

class IDateClass(Interface):
    min: Any = ...
    max: Any = ...
    resolution: Any = ...
    def today() -> None: ...
    def fromtimestamp(timestamp: Any) -> None: ...
    def fromordinal(ordinal: Any) -> None: ...

class IDate(IDateClass):
    year: Any = ...
    month: Any = ...
    day: Any = ...
    def replace(year: Any, month: Any, day: Any) -> None: ...
    def timetuple() -> None: ...
    def toordinal() -> None: ...
    def weekday() -> None: ...
    def isoweekday() -> None: ...
    def isocalendar() -> None: ...
    def isoformat() -> None: ...
    def ctime() -> None: ...
    def strftime(format: Any) -> None: ...

class IDateTimeClass(Interface):
    min: Any = ...
    max: Any = ...
    resolution: Any = ...
    def today() -> None: ...
    def now(tz: Optional[Any] = ...) -> None: ...
    def utcnow() -> None: ...
    def fromtimestamp(timestamp: Any, tz: Optional[Any] = ...) -> None: ...
    def utcfromtimestamp(timestamp: Any) -> None: ...
    def fromordinal(ordinal: Any) -> None: ...
    def combine(date: Any, time: Any) -> None: ...

class IDateTime(IDate, IDateTimeClass):
    year: Any = ...
    month: Any = ...
    day: Any = ...
    hour: Any = ...
    minute: Any = ...
    second: Any = ...
    microsecond: Any = ...
    tzinfo: Any = ...
    def date() -> None: ...
    def time() -> None: ...
    def timetz() -> None: ...
    def replace(year: Any, month: Any, day: Any, hour: Any, minute: Any, second: Any, microsecond: Any, tzinfo: Any) -> None: ...
    def astimezone(tz: Any) -> None: ...
    def utcoffset() -> None: ...
    def dst() -> None: ...
    def tzname() -> None: ...
    def timetuple() -> None: ...
    def utctimetuple() -> None: ...
    def toordinal() -> None: ...
    def weekday() -> None: ...
    def isoweekday() -> None: ...
    def isocalendar() -> None: ...
    def isoformat(sep: str = ...) -> None: ...
    def ctime() -> None: ...
    def strftime(format: Any) -> None: ...

class ITimeClass(Interface):
    min: Any = ...
    max: Any = ...
    resolution: Any = ...

class ITime(ITimeClass):
    hour: Any = ...
    minute: Any = ...
    second: Any = ...
    microsecond: Any = ...
    tzinfo: Any = ...
    def replace(hour: Any, minute: Any, second: Any, microsecond: Any, tzinfo: Any) -> None: ...
    def isoformat() -> None: ...
    def strftime(format: Any) -> None: ...
    def utcoffset() -> None: ...
    def dst() -> None: ...
    def tzname() -> None: ...

class ITZInfo(Interface):
    def utcoffset(dt: Any) -> None: ...
    def dst(dt: Any) -> None: ...
    def tzname(dt: Any) -> None: ...
    def fromutc(dt: Any) -> None: ...



# Stubs for zope.interface._zope_interface_coptimizations (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

import _interface_coptimizations
import _zope_interface_coptimizations
adapter_hooks: Any

def getObjectSpecification(*args, **kwargs) -> Any: ...
def implementedBy(*args, **kwargs) -> Any: ...
def providedBy(*args, **kwargs) -> Any: ...

class ClassProvidesBase(_interface_coptimizations.SpecificationBase):
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def __get__(self, *args, **kwargs) -> Any: ...

class InterfaceBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def __adapt__(self, *args, **kwargs) -> Any: ...
    def __call__(self, *args, **kwargs) -> Any: ...

class LookupBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def adapter_hook(self, *args, **kwargs) -> Any: ...
    def changed(self, *args, **kwargs) -> Any: ...
    def lookup(self, *args, **kwargs) -> Any: ...
    def lookup1(self, *args, **kwargs) -> Any: ...
    def lookupAll(self, *args, **kwargs) -> Any: ...
    def queryAdapter(self, *args, **kwargs) -> Any: ...
    def subscriptions(self, *args, **kwargs) -> Any: ...

class ObjectSpecificationDescriptor:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def __get__(self, *args, **kwargs) -> Any: ...

class SpecificationBase:
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def implementedBy(self, *args, **kwargs) -> Any: ...
    def isOrExtends(self, *args, **kwargs) -> Any: ...
    def providedBy(self, *args, **kwargs) -> Any: ...
    def __call__(self, *args, **kwargs) -> Any: ...

class VerifyingBase(_zope_interface_coptimizations.LookupBase):
    @classmethod
    def __init__(cls, *args, **kwargs) -> None: ...
    def adapter_hook(self, *args, **kwargs) -> Any: ...
    def changed(self, *args, **kwargs) -> Any: ...
    def lookup(self, *args, **kwargs) -> Any: ...
    def lookup1(self, *args, **kwargs) -> Any: ...
    def lookupAll(self, *args, **kwargs) -> Any: ...
    def queryAdapter(self, *args, **kwargs) -> Any: ...
    def subscriptions(self, *args, **kwargs) -> Any: ...



"""Overload example from README
"""

from typing import Optional, Union, List, overload
import zope.interface


class IAnimal(zope.interface.Interface):
    @overload
    def say() -> str:
        ...

    @overload
    def say(count: int) -> List[str]:
        ...

    def say(count: int = None) -> Union[str, List[str]]:
        pass


@zope.interface.implementer(IAnimal)
class Cow(object):
    @overload
    def say(self) -> str:
        ...

    @overload
    def say(self, count: int) -> List[str]:
        ...

    def say(self, count: int = None) -> Union[str, List[str]]:
        if count is None:
            return "Mooo"
        return ["Mooo"] * count

"""
<output>
</output>
"""



# Stubs for zope.interface.interfaces (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from zope.interface.interface import Interface

__docformat__: str

class IElement(Interface):
    __name__: Any = ...
    __doc__: Any = ...
    def getTaggedValue(tag: Any) -> None: ...
    def queryTaggedValue(tag: Any, default: Optional[Any] = ...) -> None: ...
    def getTaggedValueTags() -> None: ...
    def setTaggedValue(tag: Any, value: Any) -> None: ...

class IAttribute(IElement):
    interface: Any = ...

class IMethod(IAttribute):
    def getSignatureInfo() -> None: ...
    def getSignatureString() -> None: ...

class ISpecification(Interface):
    @staticmethod
    def providedBy(object: Any) -> bool: ...
    @staticmethod
    def implementedBy(class_: Any) -> bool: ...
    @staticmethod
    def isOrExtends(other: Any) -> bool: ...
    @staticmethod
    def extends(other: Any, strict: bool = ...) -> None: ...
    @staticmethod
    def weakref(callback: Optional[Any] = ...) -> None: ...
    def get(name: Any, default: Optional[Any] = ...) -> None: ...

class IInterface(ISpecification, IElement):
    def names(all: bool = ...) -> None: ...
    def namesAndDescriptions(all: bool = ...) -> None: ...
    def __getitem__(name: Any) -> None: ...
    def direct(name: Any) -> None: ...
    def validateInvariants(obj: Any, errors: Optional[Any] = ...) -> None: ...
    def __contains__(name: Any) -> None: ...
    def __iter__() -> None: ...
    __module__: Any = ...

class IDeclaration(ISpecification):
    def __contains__(interface: Any) -> None: ...
    def __iter__() -> None: ...
    def flattened() -> None: ...
    def __sub__(interfaces: Any) -> None: ...
    def __add__(interfaces: Any) -> None: ...
    def __nonzero__() -> None: ...

class IInterfaceDeclaration(Interface):
    def providedBy(ob: Any) -> None: ...
    def implementedBy(class_: Any) -> None: ...
    def classImplements(class_: Any, *interfaces: Any) -> None: ...
    def implementer(*interfaces: Any) -> None: ...
    def classImplementsOnly(class_: Any, *interfaces: Any) -> None: ...
    def implementer_only(*interfaces: Any) -> None: ...
    def directlyProvidedBy(object: Any) -> None: ...
    def directlyProvides(object: Any, *interfaces: Any) -> None: ...
    def alsoProvides(object: Any, *interfaces: Any) -> None: ...
    def noLongerProvides(object: Any, interface: Any) -> None: ...
    def implements(*interfaces: Any) -> None: ...
    def implementsOnly(*interfaces: Any) -> None: ...
    def classProvides(*interfaces: Any) -> None: ...
    def provider(*interfaces: Any) -> None: ...
    def moduleProvides(*interfaces: Any) -> None: ...
    def Declaration(*interfaces: Any) -> None: ...

class IAdapterRegistry(Interface):
    def register(required: Any, provided: Any, name: Any, value: Any) -> None: ...
    def registered(required: Any, provided: Any, name: Any = ...) -> None: ...
    def lookup(required: Any, provided: Any, name: str = ..., default: Optional[Any] = ...) -> None: ...
    def queryMultiAdapter(objects: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def lookup1(required: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def queryAdapter(object: Any, provided: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def adapter_hook(provided: Any, object: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def lookupAll(required: Any, provided: Any) -> None: ...
    def names(required: Any, provided: Any) -> None: ...
    def subscribe(required: Any, provided: Any, subscriber: Any, name: Any = ...) -> None: ...
    def subscriptions(required: Any, provided: Any, name: Any = ...) -> None: ...
    def subscribers(objects: Any, provided: Any, name: Any = ...) -> None: ...

class ComponentLookupError(LookupError): ...
class Invalid(Exception): ...

class IObjectEvent(Interface):
    object: Any = ...

class ObjectEvent:
    object: Any = ...
    def __init__(self, object: Any) -> None: ...

class IComponentLookup(Interface):
    adapters: Any = ...
    utilities: Any = ...
    def queryAdapter(object: Any, interface: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def getAdapter(object: Any, interface: Any, name: Any = ...) -> None: ...
    def queryMultiAdapter(objects: Any, interface: Any, name: Any = ..., default: Optional[Any] = ...) -> None: ...
    def getMultiAdapter(objects: Any, interface: Any, name: Any = ...) -> None: ...
    def getAdapters(objects: Any, provided: Any) -> None: ...
    def subscribers(objects: Any, provided: Any) -> None: ...
    def handle(*objects: Any) -> None: ...
    def queryUtility(interface: Any, name: str = ..., default: Optional[Any] = ...) -> None: ...
    def getUtilitiesFor(interface: Any) -> None: ...
    def getAllUtilitiesRegisteredFor(interface: Any) -> None: ...

class IRegistration(Interface):
    registry: Any = ...
    name: Any = ...
    info: Any = ...

class IUtilityRegistration(IRegistration):
    factory: Any = ...
    component: Any = ...
    provided: Any = ...

class _IBaseAdapterRegistration(IRegistration):
    factory: Any = ...
    required: Any = ...
    provided: Any = ...

class IAdapterRegistration(_IBaseAdapterRegistration): ...
class ISubscriptionAdapterRegistration(_IBaseAdapterRegistration): ...

class IHandlerRegistration(IRegistration):
    handler: Any = ...
    required: Any = ...

class IRegistrationEvent(IObjectEvent): ...
class RegistrationEvent(ObjectEvent): ...
class IRegistered(IRegistrationEvent): ...
class Registered(RegistrationEvent): ...
class IUnregistered(IRegistrationEvent): ...
class Unregistered(RegistrationEvent): ...

class IComponentRegistry(Interface):
    def registerUtility(component: Optional[Any] = ..., provided: Optional[Any] = ..., name: Any = ..., info: Any = ..., factory: Optional[Any] = ...) -> None: ...
    def unregisterUtility(component: Optional[Any] = ..., provided: Optional[Any] = ..., name: Any = ..., factory: Optional[Any] = ...) -> None: ...
    def registeredUtilities() -> None: ...
    def registerAdapter(factory: Any, required: Optional[Any] = ..., provided: Optional[Any] = ..., name: Any = ..., info: Any = ...) -> None: ...
    def unregisterAdapter(factory: Optional[Any] = ..., required: Optional[Any] = ..., provided: Optional[Any] = ..., name: Any = ...) -> None: ...
    def registeredAdapters() -> None: ...
    def registerSubscriptionAdapter(factory: Any, required: Optional[Any] = ..., provides: Optional[Any] = ..., name: Any = ..., info: str = ...) -> None: ...
    def unregisterSubscriptionAdapter(factory: Optional[Any] = ..., required: Optional[Any] = ..., provides: Optional[Any] = ..., name: Any = ...) -> None: ...
    def registeredSubscriptionAdapters() -> None: ...
    def registerHandler(handler: Any, required: Optional[Any] = ..., name: Any = ..., info: str = ...) -> None: ...
    def unregisterHandler(handler: Optional[Any] = ..., required: Optional[Any] = ..., name: Any = ...) -> None: ...
    def registeredHandlers() -> None: ...

class IComponents(IComponentLookup, IComponentRegistry): ...



# Stubs for zope.interface (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from zope.interface.declarations import Declaration as Declaration, alsoProvides as alsoProvides, classImplements as classImplements, classImplementsOnly as classImplementsOnly, classProvides as classProvides, directlyProvidedBy as directlyProvidedBy, directlyProvides as directlyProvides, implementedBy as implementedBy, implementer as implementer, implementer_only as implementer_only, implements as implements, implementsOnly as implementsOnly, moduleProvides as moduleProvides, noLongerProvides as noLongerProvides, providedBy as providedBy, provider as provider, named as named
from zope.interface.interface import Attribute as Attribute, Interface as Interface
from zope.exceptions import Invalid as Invalid, DoesNotImplement as DoesNotImplement, BrokenImplementation as BrokenImplementation, InvalidInterface as InvalidInterface, BadImplements as BadImplements
from zope.interface import adapter as adapter
from zope.interface import advice as advice
from zope.interface import interfaces as interfaces
from zope.interface import declarations as declarations
from zope.interface import document as document
from zope.interface import exceptions as exceptions
from zope.interface import verify as verify
from zope.interface import ro as ro



"""zope.interface.implementer accepts multiple interfaces
"""
import zope.interface

class IBookmark(zope.interface.Interface):
    def remember(url: str) -> None:
        pass

class IActionalble(zope.interface.Interface):
    def follow(times: int):
        pass

@zope.interface.implementer(IBookmark, IActionalble)
class Bookmark(object):
    pass

def main() -> None:
    bm1: IBookmark = Bookmark()
    bm1.remember(None)  # Error, string is expected

    bm2: IActionalble = Bookmark()
    bm2.follow("bad number")  # Error: integer is expected

if __name__ == '__main__':
    main()

"""
<output>
multiple_implementer.py:14: error: 'Bookmark' is missing following 'IBookmark' interface members: remember.
multiple_implementer.py:14: error: 'Bookmark' is missing following 'IActionalble' interface members: follow.
multiple_implementer.py:19: error: Argument 1 to "remember" of "IBookmark" has incompatible type "None"; expected "str"
multiple_implementer.py:22: error: Argument 1 to "follow" of "IActionalble" has incompatible type "str"; expected "int"
</output>
"""



"""Interfaces can be introspected by a set of methods"""

import zope.interface
import zope.schema
from zope.schema.fieldproperty import FieldProperty

class IBookmark(zope.interface.Interface):
    url = zope.schema.TextLine()


@zope.interface.implementer(IBookmark)
class Bookmark:
    url = FieldProperty(IBookmark['url'])


def main() -> None:
    bm = Bookmark()
    reveal_type(bm.url)
    reveal_type(IBookmark.url)
    bm.url = 'http'


if __name__ == '__main__':
    main()

"""
FieldProperty cannot find out the type of the field, unfortunately, so
types of all FieldProeprties are Any. Maybe this will get fixed some day.

<output>
interface_fieldproperty.py:18: note: Revealed type is 'Any'
interface_fieldproperty.py:19: note: Revealed type is 'builtins.str'
</output>
"""



# Stubs for zope.schema.accessors (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional
from zope.interface.interface import Method

class FieldReadAccessor(Method):
    def __provides__(self): ...
    __provides__: Any = ...
    field: Any = ...
    __doc__: Any = ...
    def __init__(self, field: Any) -> None: ...
    def getSignatureString(self): ...
    def getSignatureInfo(self): ...
    def get(self, object: Any): ...
    def query(self, object: Any, default: Optional[Any] = ...): ...
    def set(self, object: Any, value: Any) -> None: ...
    def __getattr__(self, name: Any): ...
    def bind(self, object: Any): ...

class FieldWriteAccessor(Method):
    field: Any = ...
    __doc__: Any = ...
    def __init__(self, field: Any) -> None: ...
    def getSignatureString(self): ...
    def getSignatureInfo(self): ...

def accessors(field: Any) -> None: ...



"""Something.hello has a signature, incompatible with interface
"""

import zope.interface


class ISomething(zope.interface.Interface):
    def hello(x: int, y: str) -> None:
        pass


@zope.interface.implementer(ISomething)
class Something(object):
    def hello(self, x: int, y: int) -> None:
        print(f"X: {x}, Y: {y}")


def run(smth: ISomething):
    smth.hello(1, "test")


def main() -> None:
    smth = Something()
    run(smth)


if __name__ == '__main__':
    main()

"""
<output>
incompatible_signature.py:14: error: Argument 2 of "Something" is incompatible with "hello" of supertype "ISomething"; supertype defines the argument type as "str"
incompatible_signature.py:14: note: This violates the Liskov substitution principle
incompatible_signature.py:14: note: See https://mypy.readthedocs.io/en/stable/common_issues.html#incompatible-overrides
</output>
"""
